"""
 Main program to run CBO's Financial Regulation Model
"""

from datetime import datetime
import pandas as pd
import itertools

from model import Model
from results import write_results

start_time = datetime.now()

# Read in parameters
parameter_set = pd.read_csv('..\\input_data\\parameter_set.csv')

# Instantiate the Model
model = Model(parameter_set['prob_crisis'].values)

#-----------------------------------------------------------------------------------------
# Run change Captial Requirements (CR)
#-----------------------------------------------------------------------------------------
CR_param_names = ['prob_crisis',
                  'CR_change',
                  'CR_failure',
                  'premium_change',
                  'tax_change']

parameters = pd.DataFrame()
results = pd.DataFrame()

# Convert parameters DataFrame into a list of columns 
# (need to transpose (T) the DataFrame first) 
CR_params = parameter_set[CR_param_names].T.values.tolist()

# pass in CR_param_list with * to unpack the list
CR_param_combos = itertools.product(*CR_params)

# Run model for each combination of parameters
for params in CR_param_combos:

    params_current_combo = pd.DataFrame(data = [list(params)], columns = CR_param_names)
    parameters = parameters.append(params_current_combo, ignore_index = True)
            
    param_dict = dict(zip(CR_param_names, list(params)))
   
    change_CR_results = model.run_change_CR(param_dict)
    results = results.append(change_CR_results, ignore_index = True)

write_results(results,
              parameters, 
              "change_CR_results.csv")


#-----------------------------------------------------------------------------------------
# Run eliminate Orderly Liquidation Authority (OLA) 
#-----------------------------------------------------------------------------------------
OLA_param_names = ['prob_crisis', 'OLA_fraction', 'OLA_effect']

parameters = pd.DataFrame()
results = pd.DataFrame()

# Convert parameters DataFrame into a list of columns 
# (need to transpose (T) the DataFrame first) 
OLA_params = parameter_set[OLA_param_names].T.values.tolist()

# Create the cartesian product of all possible combinations with itertools
# (pass in params with * to unpack the list)
OLA_param_combos = itertools.product(*OLA_params)

# Run model for each combination of parameters
for params in OLA_param_combos:

    params_current_combo = pd.DataFrame(data = [list(params)], columns = OLA_param_names)
    parameters = parameters.append(params_current_combo, ignore_index = True)
            
    param_dict = dict(zip(OLA_param_names, list(params)))
   
    eliminate_OLA_results = model.run_eliminate_OLA(param_dict)
    results = results.append(eliminate_OLA_results, ignore_index = True)

write_results(results,
              parameters,
             "eliminate_OLA_results.csv")


#-----------------------------------------------------------------------------------------
# Run repeal Ability-to-Repay (ATR) rules 
#-----------------------------------------------------------------------------------------
ATR_param_names = ['prob_crisis', 'ATR_effect', 'housing_tax']

parameters = pd.DataFrame()
results = pd.DataFrame()

# Convert parameters DataFrame into a list of columns 
# (need to transpose (T) the DataFrame first) 
ATR_params = parameter_set[ATR_param_names].T.values.tolist()

# Create the cartesian product of all possible combinations with itertools
# (pass in params with * to unpack the list)
ATR_param_combos = itertools.product(*ATR_params)

# Run model for each combination of parameters
for params in ATR_param_combos:

    params_current_combo = pd.DataFrame(data = [list(params)], columns = ATR_param_names)
    parameters = parameters.append(params_current_combo, ignore_index = True)
            
    param_dict = dict(zip(ATR_param_names, list(params)))
   
    repeal_ATR_results = model.run_repeal_ATR(param_dict)
    results = results.append(repeal_ATR_results, ignore_index = True)

write_results(results,
              parameters,
             "repeal_ATR_results.csv")

  
print("All done!")
print("Program took: ", datetime.now() - start_time, " to complete.")
