"""
 Module contains the Results class and a function (write_results)
"""
import pandas as pd

class Results:
    """
    A Results from a policy change
    
    Attributes:
        Description: A string containing the name of the policy experiment.
        
        budget: A pandas DataFrame containing all the budgetary effects of policy sim
            (direct, noncrisis feedback, and crisis feedback results)
        
        noncrisis_econ: A pandas DataFrame containing the macroeconomic feedback effects 
        in noncrisis times

        crisis_econ: A pandas DataFrame containing an expected value of the macroeconomic
        feedback effects arising because of changes in the likelihood of the severity of
        financial crises        
        
    Methods:
        __init__()
    """
    
    def __init__(self, description):
        self.description = description

        self.budget = pd.DataFrame()
        
        self.noncrisis_econ = pd.DataFrame()
        self.crisis_econ = pd.DataFrame()
    
#-----------------------------------------------------------------------------------------
def write_results(results,
                  parameters,
                  output_file_name,
                  output_path = '..\\output_data\\baseline_2019_05\\'):
    """
    Combine results and parameters DataFrames, then write them out to a csv file

    Parameters:
        results: A pandas DataFrame with the budget results from policy simulations
        parameters: A Pandas DataFrame with the values of the parameters used to 
            generate the results
        output_file_name: Text string with the name of the file to be written out
        output_path: Text string with path to the directory where the output file will
            be written
    
    Returns:
        Nothing.
    """

    # Horizontally concatenate lists of parameters and lists of results
    params_and_results = pd.concat([parameters, results], axis = 1)

    # Write out rounded values to csv file
    params_and_results.round(3).to_csv(output_path + output_file_name,
                                       float_format = '%.3f')
                                  
    return None

